@@ -4,7 +4,8 @@ from .core import (
     tratar_colunas_df,
     gsheet_to_df,
     plotar_tabela_aggrid_colher_selecao,
-    tratar_colunas_df_para_insercao_gsheet
+    tratar_colunas_df_para_insercao_gsheet,
+    salvar_dados_bigquery
 )
 
 __all__ = [
@@ -13,5 +14,6 @@ __all__ = [
     "tratar_colunas_df",
     "gsheet_to_df",
     "plotar_tabela_aggrid_colher_selecao",
-    "tratar_colunas_df_para_insercao_gsheet"
+    "tratar_colunas_df_para_insercao_gsheet",
+    "salvar_dados_bigquery"
 ]
