@@ -5,6 +5,10 @@ import gspread
 from google.oauth2 import service_account
 from typing import Optional
 from st_aggrid import AgGrid, GridOptionsBuilder
+from google.cloud import secretmanager, bigquery
+from pandas.api.types import is_period_dtype
+import re
+import json
 
 # Função para gerar DataFrame a partir de uma view do Phoenix
 
@@ -21,7 +25,7 @@ def gerar_df_phoenix(
         'password': password_phoenix, 
         'host': host_phoenix, 
         'database': db_phoenix
-        }
+    }
 
     conexao = mysql.connector.connect(**config)
 
@@ -150,7 +154,8 @@ def tratar_colunas_df(
 
             df[coluna] = pd.to_datetime(
                 df[coluna], 
-                format=formato_colunas_data
+                format=formato_colunas_data, 
+                errors='coerce'
             ).dt.date
 
     if lista_colunas_data_hora:
@@ -159,7 +164,8 @@ def tratar_colunas_df(
 
             df[coluna] = pd.to_datetime(
                 df[coluna], 
-                format=formato_colunas_data_hora
+                format=formato_colunas_data_hora, 
+                errors='coerce'
             )
 
     if lista_colunas_hora:
@@ -168,7 +174,8 @@ def tratar_colunas_df(
 
             df[coluna] = pd.to_datetime(
                 df[coluna], 
-                format=formato_colunas_hora
+                format=formato_colunas_hora, 
+                errors='coerce'
             ).dt.time
 
     df = adicionar_colunas_ano_mes(
@@ -189,7 +196,9 @@ def gsheet_to_df(
         lista_colunas_data: Optional[list] = None, # Lista de colunas de data
         formato_colunas_data: Optional[str] = None, # Formato das colunas de data
         lista_colunas_hora: Optional[list] = None, # Lista de colunas de hora
-        formato_colunas_hora: Optional[str] = None # Formato das colunas de hora
+        formato_colunas_hora: Optional[str] = None, # Formato das colunas de hora
+        lista_colunas_data_hora: Optional[list] = None, # Lista de colunas de data e hora
+        formato_colunas_data_hora: Optional[str] = None # Formato das colunas de data e hora
     ) -> pd.DataFrame:
 
     sheet = spread_sheet.worksheet(nome_aba)
@@ -209,7 +218,9 @@ def gsheet_to_df(
         lista_colunas_data=lista_colunas_data,
         formato_colunas_data=formato_colunas_data,
         lista_colunas_hora=lista_colunas_hora,
-        formato_colunas_hora=formato_colunas_hora
+        formato_colunas_hora=formato_colunas_hora,
+        lista_colunas_data_hora=lista_colunas_data_hora,
+        formato_colunas_data_hora=formato_colunas_data_hora
     )
 
     return df
@@ -218,11 +229,13 @@ def gsheet_to_df(
 
 def plotar_tabela_aggrid_colher_selecao(
         df: pd.DataFrame,
+        df_plotagem: pd.DataFrame,
         config_selection: str,
         use_checkbox: bool,
         header_checkbox: bool,
         autoHeight: bool,
-        autoWidth: bool
+        autoWidth: bool,
+        height: Optional[int] = None,
     ) -> list:
 
     gb = GridOptionsBuilder.from_dataframe(df)
@@ -243,12 +256,24 @@ def plotar_tabela_aggrid_colher_selecao(
 
     gridOptions = gb.build()
 
-    grid_response = AgGrid(
-        df, 
-        gridOptions=gridOptions, 
-        enable_enterprise_modules=False, 
-        fit_columns_on_grid_load=True
-    )
+    if autoHeight==False and height:
+
+        grid_response = AgGrid(
+            df_plotagem, 
+            gridOptions=gridOptions, 
+            enable_enterprise_modules=False, 
+            fit_columns_on_grid_load=True,
+            height=height
+        )
+
+    else:
+
+        grid_response = AgGrid(
+            df_plotagem, 
+            gridOptions=gridOptions, 
+            enable_enterprise_modules=False, 
+            fit_columns_on_grid_load=True
+        )
 
     selected_rows = grid_response['selected_rows']
 
@@ -273,7 +298,10 @@ def tratar_colunas_df_para_insercao_gsheet(
 
             if coluna in df_insercao.columns:
 
-                df_insercao[coluna] = pd.to_datetime(df_insercao[coluna]).dt.strftime('%Y-%m-%d').fillna('')
+                df_insercao[coluna] = pd.to_datetime(
+                    df_insercao[coluna],
+                    errors='coerce'
+                ).dt.strftime('%Y-%m-%d').fillna('')
 
     if lista_colunas_data_hora:
 
@@ -281,7 +309,10 @@ def tratar_colunas_df_para_insercao_gsheet(
 
             if coluna in df_insercao.columns:
 
-                df_insercao[coluna] = pd.to_datetime(df_insercao[coluna]).dt.strftime('%Y-%m-%d %H:%M:%S').fillna('')
+                df_insercao[coluna] = pd.to_datetime(
+                    df_insercao[coluna],
+                    errors='coerce'
+                ).dt.strftime('%Y-%m-%d %H:%M:%S').fillna('')
 
     if lista_colunas_hora:
 
@@ -289,7 +320,10 @@ def tratar_colunas_df_para_insercao_gsheet(
 
             if coluna in df_insercao.columns:
 
-                df_insercao[coluna] = pd.to_datetime(df_insercao[coluna]).dt.strftime('%H:%M:%S').fillna('')
+                df_insercao[coluna] = pd.to_datetime(
+                    df_insercao[coluna],
+                    errors='coerce'
+                ).dt.strftime('%H:%M:%S').fillna('')
 
     if lista_colunas_str:
 
@@ -319,3 +353,58 @@ def tratar_colunas_df_para_insercao_gsheet(
 
     return df_insercao
 
+# Função para inserir dataframe em um bigquery
+
+def salvar_dados_bigquery(
+        df_dados_atualizados: pd.DataFrame, 
+        tabela_destino: str,
+        project_id: str,
+        secret_id: str
+    ) -> bigquery.LoadJob:
+
+    df_dados_atualizados.replace('', pd.NA, inplace=True)
+
+    print("Autenticando no Secret Manager...")
+
+    secret_client = secretmanager.SecretManagerServiceClient()
+
+    secret_name = f"projects/{project_id}/secrets/{secret_id}/versions/latest"
+
+    response = secret_client.access_secret_version(request={"name": secret_name})
+
+    secret_payload = response.payload.data.decode("UTF-8")
+    
+    credentials_info = json.loads(secret_payload)
+
+    autenticar = service_account.Credentials.from_service_account_info(credentials_info)
+    
+    bq_client = bigquery.Client(credentials=autenticar, project=autenticar.project_id)
+
+    for col in df_dados_atualizados.columns:
+        if is_period_dtype(df_dados_atualizados[col]):
+            print(f"Convertendo coluna '{col}' de Period[M] para string.")
+            df_dados_atualizados[col] = df_dados_atualizados[col].astype(str)
+
+    df_dados_atualizados.columns = (
+        df_dados_atualizados.columns
+        .str.strip()
+        .str.normalize('NFKD')
+        .str.encode('ascii', errors='ignore')
+        .str.decode('utf-8')
+        .map(lambda x: re.sub('[^0-9a-zA-Z_]', '_', x))
+        .map(lambda x: re.sub('_+', '_', x))
+    )
+
+    print("Enviando dados para o BigQuery...")
+
+    job_config = bigquery.LoadJobConfig(
+        write_disposition="WRITE_TRUNCATE"
+    )
+
+    job = bq_client.load_table_from_dataframe(df_dados_atualizados, tabela_destino, job_config=job_config)
+
+    print("Dados enviados com sucesso!")
+
+    job.result()
+
+    return job
\ No newline at end of file
